"Use strict";
import powerbi from "powerbi-visuals-api";
import {ISelectorManager} from "../visual/visualInterfaces";
import * as d3 from "d3";
type Selection<T extends d3.BaseType> = d3.Selection<T, any, any, any>;

export default class SelectorManager implements ISelectorManager{
    public selectorContainer : Selection<HTMLElement>;
    private selector;
    
    constructor(selectorContainer:Selection<HTMLElement>){
        this.selectorContainer=selectorContainer;
    }
    public switchSelector<T extends ISelector>(c:new ()=>T){
        if(typeof(this.selector)!=typeof(c)){
            console.debug(typeof(this.selector));
            console.debug(typeof(c));
        }
        let newSelector=new c();
        this.selector?.dispose();
        this.selector=newSelector;
    }
    updateView(categories: powerbi.DataViewCategoryColumn, defaultSelect: powerbi.DataViewValueColumn, defaultStart: powerbi.DataViewValueColumn, defaultEnd: powerbi.DataViewValueColumn){
        if(!this.selector){
            this.selector=new DropDownSelector(this);
        }
        this.selector.update(categories, defaultSelect, defaultStart, defaultEnd);
    };
}

interface ISelector{
    update(categories: powerbi.DataViewCategoryColumn, defaultSelect: powerbi.DataViewValueColumn, defaultStart: powerbi.DataViewValueColumn, defaultEnd: powerbi.DataViewValueColumn);
}

abstract class Selector implements ISelector{
    constructor(manager: ISelectorManager){
        console.debug("Abstract selector:","constructor start");
        this.manager=manager;
        this.createView();
    }
    protected view:Selection<HTMLElement>;
    protected abstract createView();
    public dispose(){
        this.view?.remove();
    }
    protected readonly manager:ISelectorManager;
    public abstract update(categories: powerbi.DataViewCategoryColumn, defaultSelect: powerbi.DataViewValueColumn, defaultStart: powerbi.DataViewValueColumn, defaultEnd: powerbi.DataViewValueColumn);
}
class DropDownSelector extends Selector{
    protected createView() {
        console.debug('dropDownViewManager','createView start');
        this.view=this.manager.selectorContainer.append("select").classed("dropDown-selector",true).classed("selector",true);
        console.debug('dropDownViewManager','createView end');
    }
    public update(categories: powerbi.DataViewCategoryColumn, defaultSelect: powerbi.DataViewValueColumn, defaultStart: powerbi.DataViewValueColumn, defaultEnd: powerbi.DataViewValueColumn) {
        console.debug('dropDownViewManager','update start');
        this.view.selectAll("option").remove();
        this.view.selectAll("option").data(categories.values).enter().append("option").text(function(d){return d.toString();});
        console.debug('dropDownViewManager','update end');
    }
}
class ListSelector extends Selector{
    protected createView() {
        throw new Error("Method not implemented.");
    }
    public update(categories: powerbi.DataViewCategoryColumn, defaultSelect: powerbi.DataViewValueColumn, defaultStart: powerbi.DataViewValueColumn, defaultEnd: powerbi.DataViewValueColumn) {
        console.debug('ListSelector','updateView');
        let dropDownSelector=this.manager.selectorContainer.append("select").classed("listSelector");
    }
}
class CalendarSelector extends Selector{
    protected createView() {
        throw new Error("Method not implemented.");
    }
    public update(categories: powerbi.DataViewCategoryColumn, defaultSelect: powerbi.DataViewValueColumn, defaultStart: powerbi.DataViewValueColumn, defaultEnd: powerbi.DataViewValueColumn) {
        console.debug('calendarViewManager','updateView');
        let dropDownSelector=this.manager.selectorContainer.append("select").classed("listSelector");
    }
}